from django.conf import settings
from django.utils.six import moves


SCHEMA_URI_ERROR = 'urn:ietf:params:scim:api:messages:2.0:Error'

SCIM_CONTENT_TYPE = 'application/scim+json'

SCHEMA_URI_SERACH_REQUEST = 'urn:ietf:params:scim:api:messages:2.0:SearchRequest'

